import unittest
import Zope

from DateTime import DateTime

from Products.Silva.IContainer import IContainer
from Products.Silva.IPublication import IPublication
from Products.Silva.IRoot import IRoot

# from Products.SilvaLocalContent.LocalSilvaObject import LocalSilvaObject
from Products.SilvaLocalContent.Errors import LocaleNotSupported, LocaleAlreadyExists
from Products.SilvaLocalContent.interfaces import ILocalContainer, ILocalPublication, ILocalRoot
from Products.SilvaLocalContent.LocalDocument import LocalDocument

from test_utils import test_request, assert_equal_elements


class LocalContainerTestCase(unittest.TestCase):

            
    def setUp(self):
        get_transaction().begin()
        self.connection = Zope.DB.open()
        try:
            self.root = test_request(self.connection)

            self.acceptor = self.root.REQUEST.AcceptLanguage
            self.root.REQUEST['URL1'] = '/foo'

            self.root.manage_addProduct['SilvaLocalContent'] \
                     .manage_addLocalRoot('root', 'Root', ['en', 'la'])
            self.sroot = self.root.root

            self.sroot.manage_addProduct['SilvaLocalContent'] \
                      .manage_addLocalFolder('test_folder', u'Test Folder')
            self.test_folder = self.sroot.test_folder

            self.test_folder.manage_addProduct['SilvaLocalContent']\
                             .manage_addLocalDocument('doc', u'Test Doc')
            self.doc = self.test_folder.doc

            self.sroot.manage_addProduct['SilvaLocalContent'] \
                      .manage_addLocalPublication('test_publication', u'Test Publication')
            self.test_publication = self.sroot.test_publication
            
            self.test_publication.manage_addProduct['SilvaLocalContent'] \
                      .manage_addLocalFolder('subfolder', u'Test Subfolder')
            self.subfolder = self.test_publication.subfolder
            
        except:
            self.tearDown()
            raise


    def test_creation(self):

        self.assert_(ILocalContainer.isImplementedBy(self.test_folder))
        self.assert_(IContainer.isImplementedBy(self.test_folder))
        self.assert_(self.test_folder.implements_container())
 
        self.assert_(ILocalContainer.isImplementedBy(self.test_publication))
        self.assert_(IContainer.isImplementedBy(self.test_publication))
        self.assert_(self.test_publication.implements_container())

        self.assert_(ILocalContainer.isImplementedBy(self.sroot))
        self.assert_(IContainer.isImplementedBy(self.sroot))
        self.assert_(self.sroot.implements_container())

        self.assert_(ILocalPublication.isImplementedBy(self.test_publication))
        self.assert_(IPublication.isImplementedBy(self.test_publication))
        self.assert_(self.test_publication.implements_publication())

        self.assert_(ILocalRoot.isImplementedBy(self.sroot))
        self.assert_(IRoot.isImplementedBy(self.sroot))

        self.failIf(self.test_folder.is_published())
        default = self.test_folder.get_default()
        self.assert_(default is not None)

        self.failIf(default.is_published())
        self.assertEquals(u'Test Folder', default.get_title())

    def test_Security(self):
        # "sec_get_nearest_of_role" descents out of root
        # if not patched properly
        # check here that it does not happen
        self.failIf(self.sroot.sec_get_nearest_of_role('foobar'))
        self.failIf(self.test_folder.sec_get_nearest_of_role('foobar'))
        self.failIf(self.sroot.sec_is_closed_to_public())
        self.failIf(self.test_folder.sec_is_closed_to_public())
        # interestingly this returns false, too ...
        self.failIf(self.test_folder.sec_is_open_to_public())
        self.failIf(self.sroot.sec_is_open_to_public())

    
    def test_new_variants(self):
        folder = self.test_folder
        default = folder.get_default()
        doc = self.doc
        assert_equal_elements(['en'], default.get_available_languages())
        assert_equal_elements(['en'], default.get_supported_languages())
        self.assertEquals('en', default.get_default_language())

        assert_equal_elements(['en', 'la'], self.sroot.get_available_languages())
        assert_equal_elements(['en'], folder.get_available_languages())
        assert_equal_elements(['en', 'la'], folder.get_supported_languages())
        self.assertEquals('en', folder.get_default_language())

        assert_equal_elements(['en'], doc.get_available_languages())
        assert_equal_elements(['en'], doc.get_supported_languages())

        folder.create_language_variant('la', 'en')
        assert_equal_elements(['en', 'la'], folder.get_available_languages())
        assert_equal_elements(['en', 'la'], folder.get_supported_languages())

        assert_equal_elements(['en', 'la'], default.get_available_languages())
        assert_equal_elements(['en', 'la'], default.get_supported_languages())

        assert_equal_elements(['en'], doc.get_available_languages())
        assert_equal_elements(['en', 'la'], doc.get_supported_languages())

        doc.create_language_variant('la', 'en')
        assert_equal_elements(['en', 'la'], doc.get_available_languages())

        self.sroot.manage_addLanguage('fr')

        assert_equal_elements(['en', 'la'], folder.get_available_languages())
        assert_equal_elements(['en', 'la', 'fr'], folder.get_supported_languages())

        try:
           doc.create_language_variant('fr', 'la') 
        except LocaleNotSupported:
            pass
        else:
            self.fail()

        folder.create_language_variant('fr', 'en')
        assert_equal_elements(['en', 'la', 'fr'], default.get_supported_languages())
        assert_equal_elements(['en', 'la', 'fr'], default.get_available_languages())
        assert_equal_elements(['en', 'la', 'fr'], doc.get_supported_languages())
        doc.create_language_variant('fr', 'la')


    # test if the publication still depends on the status of the default object
    def test_publish(self):
        folder = self.test_folder
        default = folder.get_default()
        
        self.failIf(folder.is_published())
        self.assert_(self.sroot.is_delete_allowed(folder.getId() ))
        default.set_unapproved_version_publication_datetime(DateTime()-1)
        default.approve_version()

        self.assert_(folder.is_published())
        self.failIf(self.sroot.is_delete_allowed(folder.getId() ))

        self.acceptor.set_language('la')
        self.failIf(self.sroot.is_delete_allowed(folder.getId() ))

        folder.create_language_variant('la', 'en')
        self.failIf(folder.is_published())
        self.failIf(self.sroot.is_delete_allowed(folder.getId() ))

        default.set_unapproved_version_publication_datetime(DateTime()-1)
        default.approve_version()
        self.assert_(folder.is_published())
        default.close_version()
        self.failIf(folder.is_published())
        self.failIf(self.sroot.is_delete_allowed(folder.getId() ))

        self.acceptor.set_language('en')
        self.assert_(folder.is_published())
        


    def test_public_languages(self):
        # tests that the get_public_languages is properly implemented
        # to depend on the index ...

        folder = self.test_folder
        index = folder.get_default()
        
        assert_equal_elements((),
                              folder.get_public_languages())
        assert_equal_elements((),
                              folder.get_approved_languages())

        # create a second variant
        folder.create_language_variant('la', 'en')

        assert_equal_elements((),
                              folder.get_public_languages())
        assert_equal_elements((),
                              folder.get_approved_languages())

        index.set_unapproved_version_publication_datetime(DateTime()+100)
        index.approve_version()
        
        assert_equal_elements((),
                              folder.get_public_languages())
        assert_equal_elements(('en',),
                              folder.get_approved_languages())

        self.acceptor.set_language('la')
        
        assert_equal_elements((),
                              folder.get_public_languages())
        assert_equal_elements(('en',),
                              folder.get_approved_languages())

        index.set_unapproved_version_publication_datetime(DateTime()-100)
        index.approve_version()
        self.assert_(index.is_published())

        assert_equal_elements(('la',),
                              folder.get_public_languages())
        assert_equal_elements(('en',),
                              folder.get_approved_languages())

        self.acceptor.set_language('en')
        index.set_approved_version_publication_datetime(DateTime()-100)
        self.assert_(index.is_published())
        
        assert_equal_elements(('la', 'en'),
                              folder.get_public_languages())
        assert_equal_elements((),
                              folder.get_approved_languages())

        index.close_version()
        assert_equal_elements(('la',),
                              folder.get_public_languages())
        assert_equal_elements((),
                              folder.get_approved_languages())        
    

    # XXX one could test here that the "set_title" does still affect the index,
    # (and vice versa)
    # except that I do not really want that; instead both may vary
    # independently ... which is tested here

    def test_title(self):

        folder = self.test_folder
        default = folder.get_default()

        self.assertEquals('Test Folder', folder.get_title())
        self.assertEquals('Test Folder', default.get_title())

        default.set_title('Test Index')
        self.assertEquals('Test Index', default.get_title())
        self.assertEquals('Test Folder', folder.get_title())

        folder.create_language_variant('la', 'en')
        self.acceptor.set_language('la')

        default.set_title('lorem ipsum')
        self.assertEquals('lorem ipsum', default.get_title())
        self.assertEquals('Test Folder', folder.get_title())        


    def test_root_title(self):
        # just test if we can chnage the title of a root in a new variant ...
        assert_equal_elements(['en','la'], self.sroot.get_supported_languages())
        self.assertEqual('Root', self.sroot.get_title())

        self.acceptor.set_language('la')
        self.assertEqual('Root', self.sroot.get_title())

        self.sroot.set_title('Latavia root')
        self.assertEqual('Latavia root', self.sroot.get_title())
        self.acceptor.set_language('fr')
        self.assertEqual('Root', self.sroot.get_title())
        
        self.sroot.manage_addLanguage('fr')        
        self.acceptor.set_language('fr')
        self.assertEqual('Root', self.sroot.get_title())
        
        


    # test that the addables are not locale sensitive
    def test_addables(self):

        publication = self.test_publication
        self.assert_(publication.is_silva_addables_acquired())
        
        publication.set_silva_addables_allowed_in_publication([LocalDocument.meta_type])
        self.acceptor.set_language('la')
        assert_equal_elements([LocalDocument.meta_type], \
                              publication.get_silva_addables_allowed_in_publication())
        self.failIf(publication.is_silva_addables_acquired())

        assert_equal_elements([LocalDocument.meta_type], \
                              self.subfolder.get_silva_addables_allowed())
        
        # works even if language is unsupported ...
        self.acceptor.set_language('fr')
        assert_equal_elements([LocalDocument.meta_type], \
                              publication.get_silva_addables_allowed_in_publication())


    # test that one cannot copy a content, if the target
    # does not support the languages
    def _test_copy(self):
        # FIXME: this test is disabled, as it failes
        # as the current implementation does not support this

        self.sroot.service_view_registry.has_view = lambda x, y: 1
        request = self.root.REQUEST
        publication = self.test_publication
        folder = self.test_folder

        self.acceptor.set_language('la')

        # XXX debuggigng: check it is actually allowed to add things here
        #self.assert_(LocalDocument.meta_type in folder.get_silva_addables_allowed())
        #self.assert_(LocalDocument.meta_type in [ x['name'] for x in folder.get_silva_addables() ])

        publication.create_language_variant('la', 'en')
        # XXX should have a latin title, actually ...
        publication.manage_addProduct['SilvaLocalContent'].manage_addLocalDocument('test_copy','Test for copying')
        assert_equal_elements(['la'],
                              publication.test_copy.get_available_languages())

        publication.action_copy(['test_copy'], request)
        folder.action_paste(request)
        self.failIf( hasattr(folder, 'test_copy') )

        publication.test_copy.create_language_variant('en', 'la')

        publication.action_copy(['test_copy'], request)
        folder.action_paste(request)
        self.assert_( hasattr(folder, 'test_copy') )
        assert_equal_elements(['en'],
                              folder.test_copy.get_available_languages())

        
        
    def tearDown(self):
        get_transaction().abort()
        self.connection.close()

        
def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(LocalContainerTestCase, 'test'))
    return suite        
